@@ -1,14 +1,15 @@
 // Template array classes
 /*
 
-Copyright (C) 1996, 1997 John W. Eaton
+Copyright (C) 1993, 1994, 1995, 1996, 1997, 2000, 2001, 2002, 2003,
+              2004, 2005, 2006, 2007 John W. Eaton
 
 This file is part of Octave.
 
 Octave is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation; either version 3 of the License, or (at your
+option) any later version.
 
 Octave is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -16,9 +17,8 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-02110-1301, USA.
+along with Octave; see the file COPYING.  If not, see
+<http://www.gnu.org/licenses/>.
 
 */
 
@@ -110,6 +110,11 @@ protected:
 
   //--------------------------------------------------------------------
 
+public:
+
+  // !!! WARNING !!! -- these should be protected, not public.  You
+  // should not access these methods directly!
+
   void make_unique (void)
     {
       if (rep->count > 1)
@@ -130,7 +135,7 @@ protected:
 	rep->fill (val);
     }
 
-public:
+  typedef T element_type;
 
   // !!! WARNING !!! -- these should be protected, not public.  You
   // should not access these data members directly!
@@ -141,8 +146,8 @@ public:
 
 protected:
 
-  idx_vector *idx;
-  int idx_count;
+  mutable idx_vector *idx;
+  mutable int idx_count;
 
   Array (T *d, octave_idx_type n)
     : rep (new typename Array<T>::ArrayRep (d, n)), dimensions (n),
@@ -234,10 +239,10 @@ public:
 	  rep->count++;
 
 	  dimensions = a.dimensions;
-	}
 
-      idx_count = 0;
-      idx = 0;
+	  idx_count = 0;
+	  idx = 0;
+	}
 
       return *this;
     }
@@ -282,8 +287,8 @@ public:
   T range_error (const char *fcn, octave_idx_type i, octave_idx_type j, octave_idx_type k) const;
   T& range_error (const char *fcn, octave_idx_type i, octave_idx_type j, octave_idx_type k);
 
-  T range_error (const char *fcn, const Array<int>& ra_idx) const;
-  T& range_error (const char *fcn, const Array<int>& ra_idx);
+  T range_error (const char *fcn, const Array<octave_idx_type>& ra_idx) const;
+  T& range_error (const char *fcn, const Array<octave_idx_type>& ra_idx);
 
   // No checking, even for multiple references, ever.
 
@@ -296,13 +301,13 @@ public:
   T& xelem (octave_idx_type i, octave_idx_type j, octave_idx_type k) { return xelem (i, dim2()*k+j); }
   T xelem (octave_idx_type i, octave_idx_type j, octave_idx_type k) const { return xelem (i, dim2()*k+j); }
 
-  T& xelem (const Array<int>& ra_idx)
+  T& xelem (const Array<octave_idx_type>& ra_idx)
     { return xelem (compute_index (ra_idx)); }
 
-  T xelem (const Array<int>& ra_idx) const
+  T xelem (const Array<octave_idx_type>& ra_idx) const
     { return xelem (compute_index (ra_idx)); }
 
-  // XXX FIXME XXX -- would be nice to fix this so that we don't
+  // FIXME -- would be nice to fix this so that we don't
   // unnecessarily force a copy, but that is not so easy, and I see no
   // clean way to do it.
 
@@ -333,7 +338,7 @@ public:
 	return elem (i, dim2()*k+j);
     }
 
-  T& checkelem (const Array<int>& ra_idx)
+  T& checkelem (const Array<octave_idx_type>& ra_idx)
     {
       octave_idx_type i = compute_index (ra_idx);
 
@@ -353,19 +358,19 @@ public:
 
   T& elem (octave_idx_type i, octave_idx_type j, octave_idx_type k) { return elem (i, dim2()*k+j); }
 
-  T& elem (const Array<int>& ra_idx)
+  T& elem (const Array<octave_idx_type>& ra_idx)
     { return Array<T>::elem (compute_index (ra_idx)); }
 
 #if defined (BOUNDS_CHECKING)
   T& operator () (octave_idx_type n) { return checkelem (n); }
   T& operator () (octave_idx_type i, octave_idx_type j) { return checkelem (i, j); }
   T& operator () (octave_idx_type i, octave_idx_type j, octave_idx_type k) { return checkelem (i, j, k); }
-  T& operator () (const Array<int>& ra_idx) { return checkelem (ra_idx); }
+  T& operator () (const Array<octave_idx_type>& ra_idx) { return checkelem (ra_idx); }
 #else
   T& operator () (octave_idx_type n) { return elem (n); }
   T& operator () (octave_idx_type i, octave_idx_type j) { return elem (i, j); }
   T& operator () (octave_idx_type i, octave_idx_type j, octave_idx_type k) { return elem (i, j, k); }
-  T& operator () (const Array<int>& ra_idx) { return elem (ra_idx); }
+  T& operator () (const Array<octave_idx_type>& ra_idx) { return elem (ra_idx); }
 #endif
 
   T checkelem (octave_idx_type n) const
@@ -392,7 +397,7 @@ public:
 	return Array<T>::elem (i, Array<T>::dim1()*k+j);
     }
 
-  T checkelem (const Array<int>& ra_idx) const
+  T checkelem (const Array<octave_idx_type>& ra_idx) const
     {
       octave_idx_type i = compute_index (ra_idx);
 
@@ -408,19 +413,19 @@ public:
 
   T elem (octave_idx_type i, octave_idx_type j, octave_idx_type k) const { return elem (i, dim2()*k+j); }
 
-  T elem (const Array<int>& ra_idx) const
+  T elem (const Array<octave_idx_type>& ra_idx) const
     { return Array<T>::elem (compute_index (ra_idx)); }
 
 #if defined (BOUNDS_CHECKING)
   T operator () (octave_idx_type n) const { return checkelem (n); }
   T operator () (octave_idx_type i, octave_idx_type j) const { return checkelem (i, j); }
   T operator () (octave_idx_type i, octave_idx_type j, octave_idx_type k) const { return checkelem (i, j, k); }
-  T operator () (const Array<int>& ra_idx) const { return checkelem (ra_idx); }
+  T operator () (const Array<octave_idx_type>& ra_idx) const { return checkelem (ra_idx); }
 #else
   T operator () (octave_idx_type n) const { return elem (n); }
   T operator () (octave_idx_type i, octave_idx_type j) const { return elem (i, j); }
   T operator () (octave_idx_type i, octave_idx_type j, octave_idx_type k) const { return elem (i, j, k); }
-  T operator () (const Array<int>& ra_idx) const { return elem (ra_idx); }
+  T operator () (const Array<octave_idx_type>& ra_idx) const { return elem (ra_idx); }
 #endif
 
   Array<T> reshape (const dim_vector& new_dims) const;
@@ -451,9 +456,9 @@ public:
 
 public:
 
-  void resize (int n) { resize_no_fill (n); }
+  void resize (octave_idx_type n) { resize_no_fill (n); }
 
-  void resize (int n, const T& val) { resize_and_fill (n, val); }
+  void resize (octave_idx_type n, const T& val) { resize_and_fill (n, val); }
 
   void resize (const dim_vector& dv) { resize_no_fill (dv); }
 
@@ -491,9 +496,9 @@ public:
 
   void maybe_delete_dims (void);
 
-  void clear_index (void);
+  void clear_index (void) const;
 
-  void set_index (const idx_vector& i);
+  void set_index (const idx_vector& i) const;
 
   int index_count (void) const { return idx_count; }
 
@@ -511,7 +516,7 @@ public:
 
   void maybe_delete_elements (Array<idx_vector>& ra_idx, const T& rfv);
 
-  Array<T> value (void);
+  Array<T> value (void) const;
 
   Array<T> index (idx_vector& i, int resize_ok = 0,
 		  const T& rfv = resize_fill_value (T ())) const;
@@ -534,6 +539,10 @@ public:
   //  static T resize_fill_value (void) { return T (); }
 
   void print_info (std::ostream& os, const std::string& prefix) const;
+
+  // Unsafe.  This function exists to support the MEX interface.
+  // You should not use it anywhere else.
+  void *mex_get_data (void) const { return const_cast<T *> (data ()); }
 };
 
 // NOTE: these functions should be friends of the Array<T> class and
@@ -563,21 +572,21 @@ assign (Array<LT>& lhs, const Array<RT>& rhs)
   return assign (lhs, rhs, resize_fill_value (LT ()));
 }
 
-#define INSTANTIATE_ARRAY_ASSIGN(LT, RT) \
-  template int assign (Array<LT>&, const Array<RT>&, const LT&); \
-  template int assign1 (Array<LT>&, const Array<RT>&, const LT&); \
-  template int assign2 (Array<LT>&, const Array<RT>&, const LT&); \
-  template int assignN (Array<LT>&, const Array<RT>&, const LT&); \
-  template int assign (Array<LT>&, const Array<RT>&)
+#define INSTANTIATE_ARRAY_ASSIGN(LT, RT, API) \
+  template API int assign (Array<LT>&, const Array<RT>&, const LT&); \
+  template API int assign1 (Array<LT>&, const Array<RT>&, const LT&); \
+  template API int assign2 (Array<LT>&, const Array<RT>&, const LT&); \
+  template API int assignN (Array<LT>&, const Array<RT>&, const LT&); \
+  template API int assign (Array<LT>&, const Array<RT>&)
 
 
-#define INSTANTIATE_ARRAY(T) \
-  template class Array<T>; \
-  template T resize_fill_value (const T&); \
+#define INSTANTIATE_ARRAY(T, API) \
+  template class API Array<T>; \
+  template API T resize_fill_value (const T&); \
 
-#define INSTANTIATE_ARRAY_AND_ASSIGN(T) \
-  INSTANTIATE_ARRAY (T); \
-  INSTANTIATE_ARRAY_ASSIGN (T, T)
+#define INSTANTIATE_ARRAY_AND_ASSIGN(T, API) \
+  INSTANTIATE_ARRAY (T, API); \
+  INSTANTIATE_ARRAY_ASSIGN (T, T, API)
 
 #endif
 
