@@ -173,7 +173,6 @@
 ## @end deftypefn
 
 ## PKG_ADD: mark_as_command pkg
-## PKG_ADD: pkg ("load", "auto");
 
 function [local_packages, global_packages] = pkg (varargin)
   ## Installation prefix (XXX: what should these be on windows?)
@@ -758,12 +757,18 @@ function install (files, handle_deps, autoload, prefix, archprefix, verbose,
   endfor
 
   ## Add the newly installed packages to the path, so the user
-  ## can begin usings them. Only load them if they are marked autoload
+  ## can begin using them. Only load them if they are marked autoload
   if (length (descriptions) > 0)
     idx = [];
     for i = 1:length (descriptions)
       if (isautoload (descriptions(i)))
-	idx (end + 1) = i;
+	nm = descriptions{i}.name;
+	for j = 1:length (installed_pkgs_lst)
+	  if (strcmp (nm, installed_pkgs_lst{j}.name))
+	    idx (end + 1) = j;
+	    break;
+	  endif
+	endfor
       endif
     endfor
     load_packages_and_dependencies (idx, handle_deps, installed_pkgs_lst,
@@ -1020,12 +1025,21 @@ function configure_make (desc, packdir, verbose)
     src = fullfile (packdir, "src");
     ## configure
     if (exist (fullfile (src, "configure"), "file"))
+      flags = "";
+      if (isempty (getenv ("CC")))
+        flags = strcat (flags, " CC=\"", octave_config_info ("CC"), "\"");
+      endif
+      if (isempty (getenv ("CXX")))
+        flags = strcat (flags, " CXX=\"", octave_config_info ("CXX"), "\"");
+      endif
+      if (isempty (getenv ("AR")))
+        flags = strcat (flags, " AR=\"", octave_config_info ("AR"), "\"");
+      endif
+      if (isempty (getenv ("RANLIB")))
+        flags = strcat (flags, " RANLIB=\"", octave_config_info ("RANLIB"), "\"");
+      endif
       [status, output] = shell (strcat ("cd ", src, "; ./configure --prefix=\"",
-					 desc.dir, "\"",
-					 " CC=", octave_config_info ("CC"),
-					 " CXX=", octave_config_info ("CXX"),
-					 " AR=", octave_config_info ("AR"),
-					 " RANLIB=", octave_config_info ("RANLIB")));
+                                        desc.dir, "\"", flags));
       if (status != 0)
 	rm_rf (desc.dir);
 	error ("the configure script returned the following error: %s", output);
@@ -1095,7 +1109,11 @@ function configure_make (desc, packdir, verbose)
     endif
 
     ## Split into architecture dependent and independent files
-    idx = cellfun (@(x) is_architecture_dependent (x), filenames);
+    if (isempty (filenames))
+      idx = [];
+    else
+      idx = cellfun (@(x) is_architecture_dependent (x), filenames);
+    endif
     archdependent = filenames (idx);
     archindependent = filenames (!idx);
 
@@ -1838,24 +1856,24 @@ function unload_packages (files, handle_deps, local_list, global_list)
     dirs = {};
     desc = {};
     for i = 1:length (files)
-      idx = strcmp (pnames, files {i});
+      idx = strcmp (pnames, files{i});
       if (! any (idx))
 	error ("package %s is not installed", files{i});
       endif
-	dirs {end + 1} = pdirs {idx};
-        desc {end + 1} = installed_pkgs_lst {idx};
+	dirs{end+1} = pdirs{idx};
+        desc{end+1} = installed_pkgs_lst{idx};
       endfor
   endif
 
   ## Check for architecture dependent directories
   archdirs = {};
   for i = 1:length (dirs)
-    tmpdir = getarchdir (desc {i});
+    tmpdir = getarchdir (desc{i});
     if (exist (tmpdir, "dir"))
-      archdirs{end + 1} = dirs{i};
-      archdirs{end + 1} = tmpdir;
+      archdirs{end+1} = dirs{i};
+      archdirs{end+1} = tmpdir;
     else
-      archdirs{end+ 1} = dirs {i};
+      archdirs{end+1} = dirs{i};
     endif
   endfor
 
@@ -1979,7 +1997,7 @@ function newdesc = save_order (desc)
       for k = 1 : length (deps)
         for j = 1 : length (desc)
           if (strcmp (desc{j}.name, deps{k}.package))
-            tmpdesc {end + 1} = desc{j};
+            tmpdesc{end+1} = desc{j};
 	    break;
           endif
         endfor
@@ -1987,7 +2005,7 @@ function newdesc = save_order (desc)
       if (! isempty (tmpdesc))					     
         newdesc = {newdesc{:}, save_order(tmpdesc){:}, desc{i}};
       else
-        newdesc {end + 1} = desc{i};
+        newdesc{end+1} = desc{i};
       endif
     endif
   endfor
@@ -2010,11 +2028,11 @@ function load_packages_and_dependencies (idx, handle_deps, installed_pkgs_lst,
   execpath = EXEC_PATH ();
   for i = idx;
     ndir = installed_pkgs_lst{i}.dir;
-    dirs {end + 1} = ndir;
+    dirs{end+1} = ndir;
     if (exist (fullfile (dirs{end}, "bin"), "dir"))
       execpath = strcat (fullfile(dirs{end}, "bin"), ":", execpath);
     endif
-    tmpdir = getarchdir (installed_pkgs_lst {i});
+    tmpdir = getarchdir (installed_pkgs_lst{i});
     if (exist (tmpdir, "dir"))
       dirs{end + 1} = tmpdir;
       if (exist (fullfile (dirs{end}, "bin"), "dir"))
@@ -2069,7 +2087,7 @@ function dep = is_architecture_dependent (nm)
 
   dep = false;
   for i = 1 : length (archdepsuffix)
-    ext = archdepsuffix {i};
+    ext = archdepsuffix{i};
     if (ext(end) == "*")
       isglob = true;
       ext(end) = [];
@@ -2078,7 +2096,7 @@ function dep = is_architecture_dependent (nm)
     endif
     pos = findstr (nm, ext);
     if (pos)
-      if (! isglob &&  (length(nm) - pos(end) != length(ext) - 1))
+      if (! isglob && (length(nm) - pos(end) != length(ext) - 1))
 	continue;
       endif
       dep = true;
