@@ -21,23 +21,33 @@
 ## @deftypefn {Function File} {} xlabel (@var{string})
 ## @deftypefnx {Function File} {} ylabel (@var{string})
 ## @deftypefnx {Function File} {} zlabel (@var{string})
-## Specify x, y, and z axis labels for the current figure.
+## @deftypefnx {Function File} {} xlabel (@var{h}, @var{string})
+## Specify x, y, and z axis labels for the current figure. If @var{h} is
+## specified then label the axis defined by @var{h}.
 ## @seealso{plot, semilogx, semilogy, loglog, polar, mesh, contour,
 ## bar, stairs,  ylabel, title}
 ## @end deftypefn
 
 ## Author: jwe
 
-function h = xlabel (varargin)
+function retval = xlabel (varargin)
 
-  if (rem (nargin, 2) == 1)
-    if (nargout > 0)
-      h = __axis_label__ ("xlabel", varargin{:});
-    else
-      __axis_label__ ("xlabel", varargin{:});
-    endif
-  else
+  [h, varargin, nargin] = __plt_get_axis_arg__ ("xlabel", varargin{:});
+
+  if (rem (nargin, 2) != 1)
     print_usage ();
   endif
 
+  oldh = gca ();
+  unwind_protect
+    axes (h);
+    tmp = __axis_label__ ("xlabel", varargin{:});
+  unwind_protect_cleanup
+    axes (oldh);
+  end_unwind_protect
+
+  if (nargout > 0)
+    retval = h;
+  endif
+
 endfunction
