@@ -19,21 +19,30 @@
 
 ## -*- texinfo -*-
 ## @deftypefn {Function File} {} zlabel (@var{string})
+## @deftypefnx {Function File} {} zlabel (@var{h}, @var{string})
 ## See xlabel.
 ## @end deftypefn
 
 ## Author: jwe
 
-function h = zlabel (varargin)
+function retval = zlabel (varargin)
 
-  if (rem (nargin, 2) == 1)
-    if (nargout > 0)
-      h = __axis_label__ ("zlabel", varargin{:});
-    else
-      __axis_label__ ("zlabel", varargin{:});
-    endif
-  else
+  [h, varargin, nargin] = __plt_get_axis_arg__ ("zlabel", varargin{:});
+
+  if (rem (nargin, 2) != 1)
     print_usage ();
   endif
 
+  oldh = gca ();
+  unwind_protect
+    axes (h);
+    tmp = __axis_label__ ("zlabel", varargin{:});
+  unwind_protect_cleanup
+    axes (oldh);
+  end_unwind_protect
+
+  if (nargout > 0)
+    retval = h;
+  endif
+
 endfunction
