@@ -1,13 +1,13 @@
 /*
 
-Copyright (C) 2000 John W. Eaton
+Copyright (C) 2000, 2002, 2004, 2005, 2006, 2007 John W. Eaton
 
 This file is part of Octave.
 
 Octave is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation; either version 3 of the License, or (at your
+option) any later version.
 
 Octave is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -15,9 +15,8 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-02110-1301, USA.
+along with Octave; see the file COPYING.  If not, see
+<http://www.gnu.org/licenses/>.
 
 */
 
@@ -86,12 +85,16 @@ octave_comment_list : public octave_base_list<octave_comment_elt>
 {
 public:
 
+  octave_comment_list (void) { }
+
   void append (const octave_comment_elt& elt)
     { octave_base_list<octave_comment_elt>::append (elt); }
 
   void append (const std::string& s,
 	       octave_comment_elt::comment_type t = octave_comment_elt::unknown)
     { append (octave_comment_elt (s, t)); }
+
+  octave_comment_list *dup (void);
 };
 
 class
@@ -100,7 +103,7 @@ octave_comment_buffer
 public:
 
   octave_comment_buffer (void)
-    : comment_list (new octave_comment_list) { }
+    : comment_list (new octave_comment_list ()) { }
   
   static bool instance_ok (void);
 
