@@ -25,12 +25,22 @@ along with Octave; see the file COPYING.  If not, see
 #include <config.h>
 #endif
 
+#include "systime.h"
+
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif
+
+#ifdef HAVE_SYS_RESOURCE_H
+#include <sys/resource.h>
+#endif
+
 #include <cfloat>
-#include <cmath>
 
 #include <string>
 
 #include "lo-ieee.h"
+#include "lo-math.h"
 #include "str-vec.h"
 #include "quit.h"
 
@@ -47,6 +57,7 @@ along with Octave; see the file COPYING.  If not, see
 #include "pt-mat.h"
 #include "utils.h"
 #include "variables.h"
+#include "pager.h"
 
 #define ANY_ALL(FCN) \
  \
@@ -359,6 +370,112 @@ sign as @var{x}.  If @var{y} is zero, the result implementation-defined.\n\
   return retval;
 }
 
+#define NATIVE_REDUCTION_1(FCN, TYPE, DIM) \
+  (arg.is_ ## TYPE ## _type ()) \
+    { \
+      TYPE ## NDArray tmp = arg. TYPE ##_array_value (); \
+      \
+      if (! error_state) \
+        retval = tmp.FCN (DIM); \
+    }
+
+#define NATIVE_REDUCTION(FCN) \
+ \
+  octave_value retval; \
+ \
+  int nargin = args.length (); \
+ \
+  bool isnative = false; \
+  \
+  if (nargin > 1 && args(nargin - 1).is_string ()) \
+    { \
+      std::string str = args(nargin - 1).string_value (); \
+      \
+      if (! error_state) \
+	{ \
+	  if (str == "native") \
+	    isnative = true; \
+	  else if (str != "double") /* Ignore double as no single type */ \
+	    error ("sum: unrecognized string argument"); \
+          nargin --; \
+	} \
+    } \
+  \
+  if (nargin == 1 || nargin == 2) \
+    { \
+      octave_value arg = args(0); \
+ \
+      int dim = (nargin == 1 ? -1 : args(1).int_value (true) - 1); \
+ \
+      if (! error_state) \
+	{ \
+	  if (dim >= -1) \
+	    { \
+              if (isnative) \
+                { \
+                  if NATIVE_REDUCTION_1 (FCN, uint8, dim) \
+                  else if NATIVE_REDUCTION_1 (FCN, uint16, dim) \
+                  else if NATIVE_REDUCTION_1 (FCN, uint32, dim) \
+                  else if NATIVE_REDUCTION_1 (FCN, uint64, dim) \
+                  else if NATIVE_REDUCTION_1 (FCN, int8, dim) \
+                  else if NATIVE_REDUCTION_1 (FCN, int16, dim) \
+                  else if NATIVE_REDUCTION_1 (FCN, int32, dim) \
+                  else if NATIVE_REDUCTION_1 (FCN, int64, dim) \
+                  else if NATIVE_REDUCTION_1 (FCN, bool, dim) \
+                  else if (arg.is_char_matrix ()) \
+                    { \
+                       error (#FCN, ": invalid char type"); \
+                    } \
+	          else if (arg.is_complex_type ()) \
+		    { \
+		      ComplexNDArray tmp = arg.complex_array_value (); \
+                      \
+		      if (! error_state) \
+		        retval = tmp.FCN (dim); \
+		    } \
+	          else if (arg.is_real_type ()) \
+		    { \
+		      NDArray tmp = arg.array_value (); \
+                      \
+		      if (! error_state) \
+		        retval = tmp.FCN (dim); \
+		    } \
+                  else \
+		    { \
+		      gripe_wrong_type_arg (#FCN, arg); \
+		      return retval; \
+		    } \
+                } \
+	      else if (arg.is_real_type ()) \
+		{ \
+		  NDArray tmp = arg.array_value (); \
+                  \
+		  if (! error_state) \
+		    retval = tmp.FCN (dim); \
+		} \
+	      else if (arg.is_complex_type ()) \
+		{ \
+		  ComplexNDArray tmp = arg.complex_array_value (); \
+                  \
+		  if (! error_state) \
+		    retval = tmp.FCN (dim); \
+		} \
+	      else \
+		{ \
+		  gripe_wrong_type_arg (#FCN, arg); \
+		  return retval; \
+		} \
+	    } \
+	  else \
+	    error (#FCN ": invalid dimension argument = %d", dim + 1); \
+	} \
+      \
+    } \
+  else \
+    print_usage (); \
+ \
+  return retval
+
 #define DATA_REDUCTION(FCN) \
  \
   octave_value retval; \
@@ -1202,16 +1319,37 @@ Return the number of columns of @var{a}.\n\
 DEFUN (sum, args, ,
   "-*- texinfo -*-\n\
 @deftypefn {Built-in Function} {} sum (@var{x}, @var{dim})\n\
+@deftypefnx {Built-in Function} {} sum (@dots{}, 'native')\n\
 Sum of elements along dimension @var{dim}.  If @var{dim} is\n\
 omitted, it defaults to 1 (column-wise sum).\n\
 \n\
 As a special case, if @var{x} is a vector and @var{dim} is omitted,\n\
 return the sum of the elements.\n\
+\n\
+If the optional argument 'native' is given, then the sum is performed\n\
+in the same type as the original argument, rather than in the default\n\
+double type. For example\n\
+\n\
+@example\n\
+sum ([true, true])\n\
+  @result{} 2\n\
+sum ([true, true], 'native')\n\
+  @result{} true\n\
+@end example\n\
 @end deftypefn")
 {
-  DATA_REDUCTION (sum);
+  NATIVE_REDUCTION (sum);
 }
 
+/*
+
+%!assert (sum([true,true]), 2)
+%!assert (sum([true,true],'native'), true)
+%!assert (sum(int8([127,10,-20])), 117);
+%!assert (sum(int8([127,10,-20]),'native'), int8(107));
+
+*/
+
 DEFUN (sumsq, args, ,
   "-*- texinfo -*-\n\
 @deftypefn {Built-in Function} {} sumsq (@var{x}, @var{dim})\n\
@@ -1233,7 +1371,7 @@ but it uses less memory and avoids calling conj if @var{x} is real.\n\
 
 DEFUN (islogical, args, ,
   "-*- texinfo -*-\n\
-@deftypefn {Built-in Functio} {} islogical (@var{x})\n\
+@deftypefn {Built-in Function} {} islogical (@var{x})\n\
 Return true if @var{x} is a logical object.\n\
 @end deftypefn")
 {
@@ -2602,16 +2740,22 @@ a minimum of two dimensions and row vectors are left unchanged.\n\
 %!assert(norm(x,Inf), 7);
 %!assert(norm(x,-Inf), 1);
 %!assert(norm(x,"inf"), 7);
-%!assert(norm(x,"fro"), 10);
+%!assert(norm(x,"fro"), 10, -eps);
 %!assert(norm(x), 10);
 %!assert(norm([1e200, 1]), 1e200);
 %!assert(norm([3+4i, 3-4i, sqrt(31)]), 9, -4*eps);
 %!shared m
 %! m = magic (4);
 %!assert(norm(m,1), 34);
-%!assert(norm(m,2), 34);
+%!assert(norm(m,2), 34, -eps);
 %!assert(norm(m,Inf), 34);
 %!assert(norm(m,"inf"), 34);
+%!shared m2, flo, fhi
+%! m2 = [1,2;3,4];
+%! flo = 1e-300;
+%! fhi = 1e+300;
+%!assert (norm(flo*m2,"fro"), sqrt(30)*flo, -eps)
+%!assert (norm(fhi*m2,"fro"), sqrt(30)*fhi, -eps)
 */
 
 // Compute various norms of the vector X.
@@ -2631,7 +2775,7 @@ If @var{a} is a matrix:\n\
 @item @var{p} = @code{2}\n\
 Largest singular value of @var{a}.\n\
 \n\
-@item @var{p} = @code{Inf}\n\
+@item @var{p} = @code{Inf} or @code{\"inf\"}\n\
 @cindex infinity norm\n\
 Infinity norm, the largest row sum of the absolute values of @var{a}.\n\
 \n\
@@ -2643,12 +2787,15 @@ Frobenius norm of @var{a}, @code{sqrt (sum (diag (@var{a}' * @var{a})))}.\n\
 If @var{a} is a vector or a scalar:\n\
 \n\
 @table @asis\n\
-@item @var{p} = @code{Inf}\n\
+@item @var{p} = @code{Inf} or @code{\"inf\"}\n\
 @code{max (abs (@var{a}))}.\n\
 \n\
 @item @var{p} = @code{-Inf}\n\
 @code{min (abs (@var{a}))}.\n\
 \n\
+@item @var{p} = @code{\"fro\"}\n\
+Frobenius norm of @var{a}, @code{sqrt (sumsq (abs (a)))}.\n\
+\n\
 @item other\n\
 p-norm of @var{a}, @code{(sum (abs (@var{a}) .^ @var{p})) ^ (1/@var{p})}.\n\
 @end table\n\
@@ -2673,32 +2820,30 @@ p-norm of @var{a}, @code{(sum (abs (@var{a}) .^ @var{p})) ^ (1/@var{p})}.\n\
 	  if ((x_arg.rows () == 1 || x_arg.columns () == 1)
 	      && ! (x_arg.is_sparse_type () || x_arg.is_integer_type ()))
 	    {
-	      double p_val;
+	      double p_val = 2;
 
-	      octave_value p_arg;
-
-	      if (nargin == 1)
-		p_arg = 2;
-	      else
-		p_arg = args(1);
-
-	      if (p_arg.is_string ())
+	      if (nargin == 2)
 		{
-		  std::string p = args(1).string_value ();
+		  octave_value p_arg = args(1);
 
-		  if (p == "inf")
-		    p_val = octave_Inf;
-		  else if (p == "fro")
-		    p_val = -1;
+		  if (p_arg.is_string ())
+		    {
+		      std::string p = args(1).string_value ();
+
+		      if (p == "inf")
+			p_val = octave_Inf;
+		      else if (p == "fro")
+			p_val = -1;
+		      else
+			error ("norm: unrecognized norm `%s'", p.c_str ());
+		    }
 		  else
-		    error ("norm: unrecognized norm `%s'", p.c_str ());
-		}
-	      else
-		{
-		  p_val = p_arg.double_value ();
+		    {
+		      p_val = p_arg.double_value ();
 
-		  if (error_state)
-		    error ("norm: unrecognized norm value");
+		      if (error_state)
+			error ("norm: unrecognized norm value");
+		    }
 		}
 
 	      if (! error_state)
@@ -2963,6 +3108,192 @@ This function is equivalent to @code{x | y}.\n\
   BINARY_OP_DEFUN_BODY (op_el_or);
 }
 
+static double tic_toc_timestamp = -1.0;
+
+DEFUN (tic, args, nargout,
+  "-*- texinfo -*-\n\
+@deftypefn {Built-in Function} {} tic ()\n\
+@deftypefnx {Built-in Function} {} toc ()\n\
+Set or check a wall-clock timer.  Calling @code{tic} without an\n\
+output argument sets the timer.  Subsequent calls to @code{toc}\n\
+return the number of seconds since the timer was set.  For example,\n\
+\n\
+@example\n\
+tic ();\n\
+# many computations later...\n\
+elapsed_time = toc ();\n\
+@end example\n\
+\n\
+@noindent\n\
+will set the variable @code{elapsed_time} to the number of seconds since\n\
+the most recent call to the function @code{tic}.\n\
+\n\
+If called with one output argument then this function returns a scalar\n\
+of type @code{uint64} and the wall-clock timer is not started.\n\
+\n\
+@example\n\
+@group\n\
+t = tic; sleep (5); (double (tic ()) - double (t)) * 1e-6\n\
+     @result{} 5\n\
+@end group\n\
+@end example\n\
+\n\
+Nested timing with @code{tic} and @code{toc} is not supported.\n\
+Therefore @code{toc} will always return the elapsed time from the most\n\
+recent call to @code{tic}.\n\
+\n\
+If you are more interested in the CPU time that your process used, you\n\
+should use the @code{cputime} function instead.  The @code{tic} and\n\
+@code{toc} functions report the actual wall clock time that elapsed\n\
+between the calls.  This may include time spent processing other jobs or\n\
+doing nothing at all.  For example,\n\
+\n\
+@example\n\
+@group\n\
+tic (); sleep (5); toc ()\n\
+     @result{} 5\n\
+t = cputime (); sleep (5); cputime () - t\n\
+     @result{} 0\n\
+@end group\n\
+@end example\n\
+\n\
+@noindent\n\
+(This example also illustrates that the CPU timer may have a fairly\n\
+coarse resolution.)\n\
+@end deftypefn")
+{
+  octave_value retval;
+
+  int nargin = args.length ();
+
+  if (nargin != 0)
+    warning ("tic: ignoring extra arguments");
+
+  octave_time now;
+
+  double tmp = now.double_value ();
+
+  if (nargout > 0)
+    retval = static_cast<octave_uint64> (1e6 * tmp);
+  else
+    tic_toc_timestamp = tmp;
+      
+  return retval;
+}
+
+DEFUN (toc, args, nargout,
+  "-*- texinfo -*-\n\
+@deftypefn {Built-in Function} {} toc ()\n\
+See tic.\n\
+@end deftypefn")
+{
+  octave_value retval;
+
+  int nargin = args.length ();
+
+  if (nargin != 0)
+    warning ("tic: ignoring extra arguments");
+
+  if (tic_toc_timestamp < 0)
+    {
+      warning ("toc called before timer set");
+      if (nargout > 0)
+	retval = Matrix ();
+    }
+  else
+    {
+      octave_time now;
+
+      double tmp = now.double_value () - tic_toc_timestamp;
+
+      if (nargout > 0)
+	retval = tmp;
+      else
+	octave_stdout << "Elapsed time is " << tmp << " seconds.\n";
+    }
+    
+  return retval;
+}
+
+DEFUN (cputime, args, ,
+  "-*- texinfo -*-\n\
+@deftypefn {Built-in Function} {[@var{total}, @var{user}, @var{system}] =} cputime ();\n\
+Return the CPU time used by your Octave session.  The first output is\n\
+the total time spent executing your process and is equal to the sum of\n\
+second and third outputs, which are the number of CPU seconds spent\n\
+executing in user mode and the number of CPU seconds spent executing in\n\
+system mode, respectively.  If your system does not have a way to report\n\
+CPU time usage, @code{cputime} returns 0 for each of its output values.\n\
+Note that because Octave used some CPU time to start, it is reasonable\n\
+to check to see if @code{cputime} works by checking to see if the total\n\
+CPU time used is nonzero.\n\
+@end deftypefn")
+{
+  octave_value_list retval;
+  int nargin = args.length ();
+  double usr = 0.0;
+  double sys = 0.0;
+
+  if (nargin != 0)
+    warning ("tic: ignoring extra arguments");
+
+#if defined (HAVE_GETRUSAGE)
+
+  struct rusage ru;
+
+  getrusage (RUSAGE_SELF, &ru);
+
+  usr = static_cast<double> (ru.ru_utime.tv_sec) +
+    static_cast<double> (ru.ru_utime.tv_usec) * 1e-6;
+
+  sys = static_cast<double> (ru.ru_stime.tv_sec) +
+    static_cast<double> (ru.ru_stime.tv_usec) * 1e-6;
+
+#elif defined (HAVE_TIMES) && defined (HAVE_SYS_TIMES_H)
+
+  struct tms t;
+
+  times (&t);
+
+  unsigned long ticks;
+  unsigned long seconds;
+  unsigned long fraction;
+
+  ticks = t.tms_utime + t.tms_cutime;
+  fraction = ticks % HZ;
+  seconds = ticks / HZ;
+
+  usr = static_cast<double> (seconds) + static_cast<double>(fraction) /
+    static_cast<double>(HZ);
+
+  ticks = t.tms_stime + t.tms_cstime;
+  fraction = ticks % HZ;
+  seconds = ticks / HZ;
+
+  sys = static_cast<double> (seconds) + static_cast<double>(fraction) /
+    static_cast<double>(HZ);
+
+#elif defined (__WIN32__)
+
+  HANDLE hProcess = GetCurrentProcess ();
+  FILETIME ftCreation, ftExit, ftUser, ftKernel;
+  GetProcessTimes (hProcess, &ftCreation, &ftExit, &ftKernel, &ftUser);
+
+  int64_t itmp = *(reinterpret_cast<int64_t *> (&ftUser));
+  usr = static_cast<double> (itmp) * 1e-7;
+
+  itmp = *(reinterpret_cast<int64_t *> (&ftKernel));
+  sys = static_cast<double> (itmp) * 1e-7;
+
+#endif
+
+  retval (2) = sys;
+  retval (1) = usr;
+  retval (0) = sys + usr;
+
+  return retval;
+}
+
 /*
 ;;; Local Variables: ***
 ;;; mode: C++ ***
