@@ -1,13 +1,14 @@
 /*
 
-Copyright (C) 1996, 1997 John W. Eaton
+Copyright (C) 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
+              2005, 2006, 2007 John W. Eaton
 
 This file is part of Octave.
 
 Octave is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation; either version 3 of the License, or (at your
+option) any later version.
 
 Octave is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -15,9 +16,8 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Octave; see the file COPYING.  If not, write to the Free
-Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-02110-1301, USA.
+along with Octave; see the file COPYING.  If not, see
+<http://www.gnu.org/licenses/>.
 
 */
 
@@ -27,6 +27,8 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 
 #include <iostream>
 
+#include "quit.h"
+
 #include "defun.h"
 #include "error.h"
 #include "oct-obj.h"
@@ -39,15 +41,12 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 #include "ov.h"
 #include "variables.h"
 
-// If TRUE, print a warning message for empty elements in a matrix list.
-static bool Vwarn_empty_list_elements;
+#include "ov-re-sparse.h"
+#include "ov-cx-sparse.h"
 
 // The character to fill with when creating string arrays.
 char Vstring_fill_char = ' ';
 
-// Warn if concatenating double and single quoted strings.
-char Vwarn_string_concat = true;
-
 // General matrices.  This list type is much more work to handle than
 // constant matrices, but it allows us to construct matrices from
 // other matrices, variables, and functions.
@@ -65,13 +64,18 @@ private:
   public:
 
     tm_row_const_rep (void)
-      : count (1), dv (), all_str (false),
+      : count (1), dv (0, 0), all_str (false),
 	all_sq_str (false), all_dq_str (false),
-	some_str (false), is_cmplx (false), all_mt (true), ok (false) { }
+	some_str (false), all_real (false), all_cmplx (false),
+	all_mt (true), any_sparse (false),
+	class_nm (octave_base_value::static_class_name ()), ok (false)
+    { }
 
     tm_row_const_rep (const tree_argument_list& row)
-      : count (1), dv (), all_str (false), all_sq_str (false),
-	some_str (false), is_cmplx (false), all_mt (true), ok (false)
+      : count (1), dv (0, 0), all_str (false), all_sq_str (false),
+	some_str (false), all_real (false), all_cmplx (false),
+	all_mt (true), any_sparse (false),
+	class_nm (octave_base_value::static_class_name ()), ok (false)
     { init (row); }
 
     ~tm_row_const_rep (void) { }
@@ -84,8 +88,12 @@ private:
     bool all_sq_str;
     bool all_dq_str;
     bool some_str;
-    bool is_cmplx;
+    bool all_real;
+    bool all_cmplx;
     bool all_mt;
+    bool any_sparse;
+
+    std::string class_nm;
 
     bool ok;
 
@@ -145,11 +153,10 @@ public:
       delete rep;
   }
 
-  octave_idx_type rows (void)
-  { return (rep->dv.length () > 0 ? rep->dv(0) : 0); }
+  octave_idx_type rows (void) { return rep->dv(0); }
+  octave_idx_type cols (void) { return rep->dv(1); }
 
-  octave_idx_type cols (void)
-  { return (rep->dv.length () > 1 ? rep->dv(1) : 0); }
+  bool empty (void) const { return rep->empty (); }
 
   dim_vector dims (void) { return rep->dv; }
 
@@ -157,8 +164,12 @@ public:
   bool all_sq_strings_p (void) const { return rep->all_sq_str; }
   bool all_dq_strings_p (void) const { return rep->all_dq_str; }
   bool some_strings_p (void) const { return rep->some_str; }
-  bool complex_p (void) const { return rep->is_cmplx; }
+  bool all_real_p (void) const { return rep->all_real; }
+  bool all_complex_p (void) const { return rep->all_cmplx; }
   bool all_empty_p (void) const { return rep->all_mt; }
+  bool any_sparse_p (void) const { return rep->any_sparse; }
+
+  std::string class_name (void) const { return rep->class_nm; }
 
   operator bool () const { return (rep && rep->ok); }
 
@@ -173,6 +184,69 @@ private:
   tm_row_const_rep *rep;
 };
 
+static std::string
+get_concat_class (const std::string& c1, const std::string& c2)
+{
+  std::string retval = octave_base_value::static_class_name ();
+
+  if (c1 == c2)
+    retval = c1;
+  else
+    {
+      bool c1_is_int = (c1 == "int8" || c1 == "uint8"
+			|| c1 == "int16" || c1 == "uint16"
+			|| c1 == "int32" || c1 == "uint32"
+			|| c1 == "int64" || c1 == "uint64");
+      bool c2_is_int = (c2 == "int8" || c2 == "uint8"
+			|| c2 == "int16" || c2 == "uint16"
+			|| c2 == "int32" || c2 == "uint32"
+			|| c2 == "int64" || c2 == "uint64");
+
+      bool c1_is_char = (c1 == "char");
+      bool c2_is_char = (c2 == "char");
+
+      bool c1_is_double = (c1 == "double");
+      bool c2_is_double = (c2 == "double");
+
+      bool c1_is_single = (c1 == "single");
+      bool c2_is_single = (c2 == "single");
+
+      bool c1_is_logical = (c1 == "logical");
+      bool c2_is_logical = (c2 == "logical");
+
+      bool c1_is_built_in_type
+	= (c1_is_int || c1_is_char || c1_is_double || c1_is_single
+	   || c1_is_logical);
+
+      bool c2_is_built_in_type
+	= (c2_is_int || c2_is_char ||  c2_is_double || c2_is_single
+	   || c2_is_logical);
+
+      // Order is important here...
+
+      if (c1_is_char && c2_is_built_in_type)
+	retval = c1;
+      else if (c2_is_char && c1_is_built_in_type)
+	retval = c2;
+      else if (c1_is_int && c2_is_built_in_type)
+	retval = c1;
+      else if (c2_is_int && c1_is_built_in_type)
+	retval = c2;
+      else if (c1_is_single && c2_is_built_in_type)
+	retval = c1;
+      else if (c2_is_single && c1_is_built_in_type)
+	retval = c2;
+      else if (c1_is_double && c2_is_built_in_type)
+	retval = c1;
+      else if (c2_is_double && c1_is_built_in_type)
+	retval = c2;
+      else if (c1_is_logical && c2_is_logical)
+	retval = c1;
+    }
+
+  return retval;    
+}
+
 bool
 tm_row_const::tm_row_const_rep::do_init_element (tree_expression *elt,
 						 const octave_value& val,
@@ -181,9 +255,11 @@ tm_row_const::tm_row_const_rep::do_init_element (tree_expression *elt,
   octave_idx_type this_elt_nr = val.rows ();
   octave_idx_type this_elt_nc = val.columns ();
 
+  std::string this_elt_class_nm = val.class_name ();
+
   dim_vector this_elt_dv = val.dims ();
 
-  if (!this_elt_dv.all_zero ())
+  if (! this_elt_dv.all_zero ())
     {
       all_mt = false;
 
@@ -191,6 +267,8 @@ tm_row_const::tm_row_const_rep::do_init_element (tree_expression *elt,
 	{
 	  first_elem = false;
 
+	  class_nm = this_elt_class_nm;
+
 	  dv.resize (this_elt_dv.length ());
 	  for (int i = 2; i < dv.length (); i++)
 	    dv.elem (i) = this_elt_dv.elem (i);
@@ -201,6 +279,8 @@ tm_row_const::tm_row_const_rep::do_init_element (tree_expression *elt,
 	}
       else
 	{
+	  class_nm = get_concat_class (class_nm, this_elt_class_nm);
+
 	  int len = (this_elt_dv.length () < dv.length ()
 		     ? this_elt_dv.length () : dv.length ());
 
@@ -238,7 +318,7 @@ tm_row_const::tm_row_const_rep::do_init_element (tree_expression *elt,
       dv.elem (1) = dv.elem (1) + this_elt_nc;
 
     }
-  else if (Vwarn_empty_list_elements)
+  else
     eval_warning ("empty matrix found in matrix list",
 		  elt->line (), elt->column ());
 
@@ -256,8 +336,14 @@ tm_row_const::tm_row_const_rep::do_init_element (tree_expression *elt,
   if (! some_str && val.is_string ())
     some_str = true;
 
-  if (! is_cmplx && val.is_complex_type ())
-    is_cmplx = true;
+  if (all_real && ! val.is_real_type ())
+    all_real = false;
+
+  if (all_cmplx && ! (val.is_complex_type () || val.is_real_type ()))
+    all_cmplx = false;
+
+  if (!any_sparse && val.is_sparse_type ())
+    any_sparse = true;
 
   return true;
 }
@@ -268,6 +354,9 @@ tm_row_const::tm_row_const_rep::init (const tree_argument_list& row)
   all_str = true;
   all_sq_str = true;
   all_dq_str = true;
+  all_real = true;
+  all_cmplx = true;
+  any_sparse = false;
 
   bool first_elem = true;
 
@@ -275,6 +364,8 @@ tm_row_const::tm_row_const_rep::init (const tree_argument_list& row)
        p != row.end ();
        p++)
     {
+      OCTAVE_QUIT;
+
       tree_expression *elt = *p;
 
       octave_value tmp = elt->rvalue ();
@@ -289,6 +380,8 @@ tm_row_const::tm_row_const_rep::init (const tree_argument_list& row)
 
 	      for (octave_idx_type i = 0; i < tlst.length (); i++)
 		{
+		  OCTAVE_QUIT;
+
 		  if (! do_init_element (elt, tlst(i), first_elem))
 		    goto done;
 		}
@@ -331,9 +424,10 @@ tm_row_const::tm_row_const_rep::eval_warning (const char *msg, int l,
 					      int c) const
 {
   if (l == -1 && c == -1)
-    ::warning ("%s", msg);
+    warning_with_id ("Octave:empty-list-elements", "%s", msg);
   else
-    ::warning ("%s near line %d, column %d", msg, l, c);
+    warning_with_id ("Octave:empty-list-elements",
+		     "%s near line %d, column %d", msg, l, c);
 }
 
 class
@@ -342,14 +436,16 @@ tm_const : public octave_base_list<tm_row_const>
 public:
 
   tm_const (const tree_matrix& tm)
-    : dv (), all_str (false), all_sq_str (false), all_dq_str (false),
-      some_str (false), is_cmplx (false), all_mt (true), ok (false)
-      { init (tm); }
+    : dv (0, 0), all_str (false), all_sq_str (false), all_dq_str (false),
+      some_str (false), all_real (false), all_cmplx (false),
+      all_mt (true), any_sparse (false),
+      class_nm (octave_base_value::static_class_name ()), ok (false)
+  { init (tm); }
 
   ~tm_const (void) { }
 
-  octave_idx_type rows (void) const { return (dv.length () > 0 ? dv.elem (0) : 0); }
-  octave_idx_type cols (void) const { return (dv.length () > 1 ? dv.elem (1) : 0); }
+  octave_idx_type rows (void) const { return dv.elem (0); }
+  octave_idx_type cols (void) const { return dv.elem (1); }
 
   dim_vector dims (void) const { return dv; }
 
@@ -357,8 +453,12 @@ public:
   bool all_sq_strings_p (void) const { return all_sq_str; }
   bool all_dq_strings_p (void) const { return all_dq_str; }
   bool some_strings_p (void) const { return some_str; }
-  bool complex_p (void) const { return is_cmplx; }
+  bool all_real_p (void) const { return all_real; }
+  bool all_complex_p (void) const { return all_cmplx; }
   bool all_empty_p (void) const { return all_mt; }
+  bool any_sparse_p (void) const { return any_sparse; }
+
+  std::string class_name (void) const { return class_nm; }
 
   operator bool () const { return ok; }
 
@@ -370,8 +470,12 @@ private:
   bool all_sq_str;
   bool all_dq_str;
   bool some_str;
-  bool is_cmplx;
+  bool all_real;
+  bool all_cmplx;
   bool all_mt;
+  bool any_sparse;
+
+  std::string class_nm;
 
   bool ok;
 
@@ -390,6 +494,9 @@ tm_const::init (const tree_matrix& tm)
   all_str = true;
   all_sq_str = true;
   all_dq_str = true;
+  all_real = true;
+  all_cmplx = true;
+  any_sparse = false;
 
   bool first_elem = true;
 
@@ -400,11 +507,13 @@ tm_const::init (const tree_matrix& tm)
 
   for (tree_matrix::const_iterator p = tm.begin (); p != tm.end (); p++)
     {
+      OCTAVE_QUIT;
+
       tree_argument_list *elt = *p;
 
       tm_row_const tmp (*elt);
 
-      if (tmp)
+      if (tmp && ! tmp.empty ())
 	{
 	  if (all_str && ! tmp.all_strings_p ())
 	    all_str = false;
@@ -418,12 +527,18 @@ tm_const::init (const tree_matrix& tm)
 	  if (! some_str && tmp.some_strings_p ())
 	    some_str = true;
 
-	  if (! is_cmplx && tmp.complex_p ())
-	    is_cmplx = true;
+	  if (all_real && ! tmp.all_real_p ())
+	    all_real = false;
+
+	  if (all_cmplx && ! tmp.all_complex_p ())
+	    all_cmplx = false;
 
 	  if (all_mt && ! tmp.all_empty_p ())
 	    all_mt = false;
 
+	  if (!any_sparse && tmp.any_sparse_p ())
+	    any_sparse = true;
+
 	  append (tmp);
 	}
       else
@@ -434,11 +549,15 @@ tm_const::init (const tree_matrix& tm)
     {
       for (iterator p = begin (); p != end (); p++)
 	{
+	  OCTAVE_QUIT;
+
 	  tm_row_const elt = *p;
 
 	  octave_idx_type this_elt_nr = elt.rows ();
 	  octave_idx_type this_elt_nc = elt.cols ();
 
+	  std::string this_elt_class_nm = elt.class_name ();
+
 	  dim_vector this_elt_dv = elt.dims ();
 
 	  if (!this_elt_dv.all_zero ())
@@ -449,6 +568,8 @@ tm_const::init (const tree_matrix& tm)
 		{
 		  first_elem = false;
 
+		  class_nm = this_elt_class_nm;
+
 		  dv.resize (this_elt_dv.length ());
 		  for (int i = 2; i < dv.length (); i++)
 		    dv.elem (i) = this_elt_dv.elem (i);
@@ -459,11 +580,15 @@ tm_const::init (const tree_matrix& tm)
 		}
 	      else if (all_str)
 		{
+		  class_nm = get_concat_class (class_nm, this_elt_class_nm);
+
 		  if (this_elt_nc > cols ())
 		    dv.elem (1) = this_elt_nc;
 		}
 	      else
 		{
+		  class_nm = get_concat_class (class_nm, this_elt_class_nm);
+
 		  bool get_out = false;
 		  int len = (this_elt_dv.length () < dv.length ()
 			     ? this_elt_dv.length () : dv.length ());
@@ -508,8 +633,9 @@ tm_const::init (const tree_matrix& tm)
 		}
 	      dv.elem (0) = dv.elem (0) + this_elt_nr;
 	    }
-	  else if (Vwarn_empty_list_elements)
-	    warning ("empty matrix found in matrix list");
+	  else
+	    warning_with_id ("Octave:empty-list-elements",
+			     "empty matrix found in matrix list");
 	}
     }
 
@@ -531,6 +657,8 @@ tree_matrix::has_magic_end (void) const
 {
   for (const_iterator p = begin (); p != end (); p++)
     {
+      OCTAVE_QUIT;
+
       tree_argument_list *elt = *p;
 
       if (elt && elt->has_magic_end ())
@@ -545,6 +673,8 @@ tree_matrix::all_elements_are_constant (void) const
 {
   for (const_iterator p = begin (); p != end (); p++)
     {
+      OCTAVE_QUIT;
+
       tree_argument_list *elt = *p;
 
       if (! elt->all_elements_are_constant ())
@@ -572,213 +702,297 @@ tree_matrix::rvalue (int nargout)
 static void
 maybe_warn_string_concat (bool all_dq_strings_p, bool all_sq_strings_p)
 {
-  if (Vwarn_string_concat && ! (all_dq_strings_p || all_sq_strings_p))
-    ::warning ("concatenation of different character string types may have unintended consequences");
+  if (! (all_dq_strings_p || all_sq_strings_p))
+    warning_with_id ("Octave:string-concat",
+		     "concatenation of different character string types may have unintended consequences");
 }
 
+#define SINGLE_TYPE_CONCAT(TYPE, EXTRACTOR) \
+  do \
+    { \
+      int dv_len = dv.length (); \
+      Array<octave_idx_type> ra_idx (dv_len > 1 ? dv_len : 2, 0); \
+ \
+      for (tm_const::iterator p = tmp.begin (); p != tmp.end (); p++) \
+	{ \
+          OCTAVE_QUIT; \
+ \
+	  tm_row_const row = *p; \
+ \
+	  for (tm_row_const::iterator q = row.begin (); \
+	       q != row.end (); \
+	       q++) \
+	    { \
+	      OCTAVE_QUIT; \
+ \
+	      TYPE ra = q->EXTRACTOR (); \
+ \
+	      if (! error_state) \
+		{ \
+		  result.insert (ra, ra_idx); \
+ \
+		  if (! error_state) \
+		    ra_idx(1) += ra.columns (); \
+		  else \
+		    goto done; \
+		} \
+	      else \
+		goto done; \
+	    } \
+ \
+	  ra_idx(0) += row.rows (); \
+	  ra_idx(1) = 0; \
+	} \
+    } \
+ while (0)
+
+#define DO_SINGLE_TYPE_CONCAT(TYPE, EXTRACTOR) \
+  do \
+    { \
+      TYPE result (dv); \
+ \
+      SINGLE_TYPE_CONCAT(TYPE, EXTRACTOR); \
+ \
+      retval = result; \
+    } \
+  while (0)
+
 octave_value
 tree_matrix::rvalue (void)
 {
-  octave_value retval;
+  octave_value retval = Matrix ();
 
   bool all_strings_p = false;
   bool all_sq_strings_p = false;
   bool all_dq_strings_p = false;
   bool all_empty_p = false;
+  bool all_real_p = false;
+  bool all_complex_p = false;
+  bool any_sparse_p = false;
   bool frc_str_conv = false;
 
   tm_const tmp (*this);
 
-  if (tmp)
+  if (tmp && ! tmp.empty ())
     {
       dim_vector dv = tmp.dims ();
       all_strings_p = tmp.all_strings_p ();
       all_sq_strings_p = tmp.all_sq_strings_p ();
       all_dq_strings_p = tmp.all_dq_strings_p ();
       all_empty_p = tmp.all_empty_p ();
+      all_real_p = tmp.all_real_p ();
+      all_complex_p = tmp.all_complex_p ();
+      any_sparse_p = tmp.any_sparse_p ();
       frc_str_conv = tmp.some_strings_p ();
 
-      // XXX FIXME XX
-      // The previous version of this code obtained the return type and 
-      // initialized an array of the correct type. However the return type
-      // is now built-up from the return types of do_cat_op. Should we special
-      // case the situation where there are only NDArray and ComplexNDArray
-      // elements, or things like boolMatrix that widen to them, and do the
-      // correct initialization? How to do this? Will it be faster? Check against
-      // version 2.1.57
+      // Try to speed up the common cases.
 
+      std::string result_type = tmp.class_name ();
 
-      // The line below might seem crazy, since we take a copy
-      // of the first argument, resize it to be empty and then resize
-      // it to be full. This is done since it means that there is no
-      // recopying of data, as would happen if we used a single resize.
-      // It should be noted that resize operation is also significantly 
-      // slower than the do_cat_op function, so it makes sense to have an
-      // empty matrix and copy all data.
-      //
-      // We might also start with a empty octave_value using
-      //    ctmp = octave_value_typeinfo::lookup_type
-      //          (tmp.begin() -> begin() -> type_name());
-      // and then directly resize. However, for some types there might be
-      // some additional setup needed, and so this should be avoided.
-
-      octave_value ctmp;
-      if (all_strings_p)
+      if (result_type == "double")
+	{
+	  if (any_sparse_p)
+	    {	    
+	      if (all_real_p)
+		DO_SINGLE_TYPE_CONCAT (SparseMatrix, sparse_matrix_value);
+	      else
+		DO_SINGLE_TYPE_CONCAT (SparseComplexMatrix, sparse_complex_matrix_value);
+	    }
+	  else
+	    {
+	      if (all_real_p)
+		DO_SINGLE_TYPE_CONCAT (NDArray, array_value);
+	      else
+		DO_SINGLE_TYPE_CONCAT (ComplexNDArray, complex_array_value);
+	    }
+	}
+#if 0
+      else if (result_type == "single")
+#endif
+      else if (result_type == "char")
 	{
 	  char type = all_sq_strings_p ? '\'' : '"';
 
 	  maybe_warn_string_concat (all_dq_strings_p, all_sq_strings_p);
 
-	  if (all_empty_p)
-	    ctmp = octave_value (charNDArray (), true, type);
-	  else
-	    ctmp = octave_value (charNDArray (dv, Vstring_fill_char),
-				 true, type);
+	  charNDArray result (dv, Vstring_fill_char);
+
+	  SINGLE_TYPE_CONCAT (charNDArray, char_array_value);
+
+	  retval = octave_value (result, true, type);
 	}
+      else if (result_type == "logical")
+	{
+	  if (any_sparse_p)
+	    DO_SINGLE_TYPE_CONCAT (SparseBoolMatrix, sparse_bool_matrix_value);
+	  else
+	    DO_SINGLE_TYPE_CONCAT (boolNDArray, bool_array_value);
+	}
+      else if (result_type == "int8")
+	DO_SINGLE_TYPE_CONCAT (int8NDArray, int8_array_value);
+      else if (result_type == "int16")
+	DO_SINGLE_TYPE_CONCAT (int16NDArray, int16_array_value);
+      else if (result_type == "int32")
+	DO_SINGLE_TYPE_CONCAT (int32NDArray, int32_array_value);
+      else if (result_type == "int64")
+	DO_SINGLE_TYPE_CONCAT (int64NDArray, int64_array_value);
+      else if (result_type == "uint8")
+	DO_SINGLE_TYPE_CONCAT (uint8NDArray, uint8_array_value);
+      else if (result_type == "uint16")
+	DO_SINGLE_TYPE_CONCAT (uint16NDArray, uint16_array_value);
+      else if (result_type == "uint32")
+	DO_SINGLE_TYPE_CONCAT (uint32NDArray, uint32_array_value);
+      else if (result_type == "uint64")
+	DO_SINGLE_TYPE_CONCAT (uint64NDArray, uint64_array_value);
       else
 	{
+	  // The line below might seem crazy, since we take a copy of
+	  // the first argument, resize it to be empty and then resize
+	  // it to be full. This is done since it means that there is
+	  // no recopying of data, as would happen if we used a single
+	  // resize.  It should be noted that resize operation is also
+	  // significantly slower than the do_cat_op function, so it
+	  // makes sense to have an empty matrix and copy all data.
+	  //
+	  // We might also start with a empty octave_value using
+	  //
+	  //    ctmp = octave_value_typeinfo::lookup_type
+	  //          (tmp.begin() -> begin() -> type_name());
+	  //
+	  // and then directly resize. However, for some types there
+	  // might be some additional setup needed, and so this should
+	  // be avoided.
+
+	  octave_value ctmp;
+
 	  // Find the first non-empty object
-	  for (tm_const::iterator p = tmp.begin (); p != tmp.end (); p++)
+
+	  if (any_sparse_p)
 	    {
-	      tm_row_const row = *p;
-	      for (tm_row_const::iterator q = row.begin (); 
-		   q != row.end (); q++)
+	      // Start with sparse matrix to avoid issues memory issues
+	      // with things like [ones(1,4),sprandn(1e8,4,1e-4)]
+	      if (all_real_p)
+		ctmp = octave_sparse_matrix ().resize (dv); 
+	      else
+		ctmp = octave_sparse_complex_matrix ().resize (dv); 
+	    }
+	  else
+	    {
+	      for (tm_const::iterator p = tmp.begin (); p != tmp.end (); p++)
 		{
-		  ctmp = *q;
-		  if (! ctmp.all_zero_dims ())
-		    goto found_non_empty;
+		  OCTAVE_QUIT;
+
+		  tm_row_const row = *p;
+
+		  for (tm_row_const::iterator q = row.begin (); 
+		       q != row.end (); q++)
+		    {
+		      OCTAVE_QUIT;
+
+		      ctmp = *q;
+
+		      if (! ctmp.all_zero_dims ())
+			goto found_non_empty;
+		    }
 		}
+
+	      ctmp = (*(tmp.begin() -> begin()));
+
+	    found_non_empty:
+
+	      if (! all_empty_p)
+		ctmp = ctmp.resize (dim_vector (0,0)).resize (dv);
 	    }
 
-	  ctmp = (*(tmp.begin() -> begin()));
-
-	found_non_empty:
-	  if (! all_empty_p)
-	    ctmp = ctmp.resize (dim_vector (0,0)).resize (dv);
-	}
-
-      if (error_state)
-	goto done;
-
-      // Now, extract the values from the individual elements and
-      // insert them in the result matrix.
-      int dv_len = dv.length ();
-      Array<int> ra_idx (dv_len > 1 ? dv_len : 2, 0);
-      for (tm_const::iterator p = tmp.begin (); p != tmp.end (); p++)
-	{
-	  tm_row_const row = *p;
-	  for (tm_row_const::iterator q = row.begin (); q != row.end (); q++)
+	  if (! error_state)
 	    {
-	      octave_value elt = *q;
+	      // Now, extract the values from the individual elements and
+	      // insert them in the result matrix.
 
-	      ctmp = do_cat_op (ctmp, elt, ra_idx);
-	      if (error_state)
-		goto done;
-	      ra_idx (1) += elt.columns ();
+	      int dv_len = dv.length ();
+	      Array<octave_idx_type> ra_idx (dv_len > 1 ? dv_len : 2, 0);
+
+	      for (tm_const::iterator p = tmp.begin (); p != tmp.end (); p++)
+		{
+		  OCTAVE_QUIT;
+
+		  tm_row_const row = *p;
+
+		  for (tm_row_const::iterator q = row.begin ();
+		       q != row.end ();
+		       q++)
+		    {
+		      OCTAVE_QUIT;
+
+		      octave_value elt = *q;
+
+		      ctmp = do_cat_op (ctmp, elt, ra_idx);
+
+		      if (error_state)
+			goto done;
+
+		      ra_idx (1) += elt.columns ();
+		    }
+
+		  ra_idx (0) += row.rows ();
+		  ra_idx (1) = 0;
+		}
+
+	      retval = ctmp;
+
+	      if (frc_str_conv && ! retval.is_string ())
+		retval = retval.convert_to_str ();
 	    }
-	  ra_idx (0) += row.rows ();
-	  ra_idx (1) = 0;
 	}
-      retval = ctmp;
-      if (frc_str_conv && ! retval.is_string ())
-	retval = retval.convert_to_str ();
     }
 
 done:
   return retval;
 }
 
+tree_expression *
+tree_matrix::dup (symbol_table *sym_tab)
+{
+  tree_matrix *new_matrix = new tree_matrix (0, line (), column ());
+
+  for (iterator p = begin (); p != end (); p++)
+    {
+      tree_argument_list *elt = *p;
+
+      new_matrix->append (elt ? elt->dup (sym_tab) : 0);
+    }
+
+  new_matrix->copy_base (*this);
+
+  return new_matrix;
+}
+
 void
 tree_matrix::accept (tree_walker& tw)
 {
   tw.visit_matrix (*this);
 }
 
-static int
-warn_empty_list_elements (void)
-{
-  Vwarn_empty_list_elements = check_preference ("warn_empty_list_elements");
-
-  return 0;
-}
-
-static int
-warn_string_concat (void)
-{
-  Vwarn_string_concat = check_preference ("warn_string_concat");
-
-  return 0;
-}
-
-static int
-string_fill_char (void)
-{
-  int status = 0;
-
-  std::string s = builtin_string_variable ("string_fill_char");
-
-  switch (s.length ())
-    {
-    case 1:
-      Vstring_fill_char = s[0];
-      break;
-
-    case 0:
-      Vstring_fill_char = '\0';
-      break;
-
-    default:
-      warning ("string_fill_char must be a single character");
-      status = -1;
-      break;
-    }
-
-  return status;
-}
-
-void
-symbols_of_pt_mat (void)
-{
-  DEFVAR (string_fill_char, " ", string_fill_char,
-    "-*- texinfo -*-\n\
-@defvr {Built-in Variable} string_fill_char\n\
-The value of this variable is used to pad all strings in a string matrix\n\
-to the same length.  It should be a single character.  The default value\n\
-is @code{\" \"} (a single space).  For example,\n\
+DEFUN (string_fill_char, args, nargout,
+  "-*- texinfo -*-\n\
+@deftypefn {Built-in Function} {@var{val} =} string_fill_char ()\n\
+@deftypefnx {Built-in Function} {@var{old_val} =} string_fill_char (@var{new_val})\n\
+Query or set the internal variable used to pad all rows of a character\n\
+matrix to the same length.  It must be a single character.  The default\n\
+value is @code{\" \"} (a single space).  For example,\n\
 \n\
 @example\n\
 @group\n\
-string_fill_char = \"X\";\n\
+string_fill_char (\"X\");\n\
 [ \"these\"; \"are\"; \"strings\" ]\n\
      @result{} \"theseXX\"\n\
         \"areXXXX\"\n\
         \"strings\"\n\
 @end group\n\
 @end example\n\
-@end defvr");
-
-  DEFVAR (warn_empty_list_elements, false, warn_empty_list_elements,
-    "-*- texinfo -*-\n\
-@defvr {Built-in Variable} warn_empty_list_elements\n\
-If the value of @code{warn_empty_list_elements} is nonzero, print a\n\
-warning when an empty matrix is found in a matrix list.  For example,\n\
-\n\
-@example\n\
-a = [1, [], 3, [], 5]\n\
-@end example\n\
-\n\
-@noindent\n\
-The default value is 0.\n\
-@end defvr");
-
-  DEFVAR (warn_string_concat, true, warn_string_concat,
-    "-*- texinfo -*-\n\
-@defvr {Built-in Variable} warn_string_concat\n\
-If the value of @code{warn_string_concat} is nonzero, print a\n\
-warning when concatenating a mixture of double and single quoted strings.\n\
-The default value is 1.\n\
-@end defvr");
-
+@end deftypefn")
+{
+  return SET_INTERNAL_VARIABLE (string_fill_char);
 }
 
 /*
